﻿//css_ref System.Data;
//css_ref System.Transactions;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Sbo.Logex.Servofarma.dll;

using System;
using System.Transactions;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Servofarma;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Diagnostics;
using Produmex.Sbo.Logex.Servofarma.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Servofarma.BusinessObjects;
using System.Reflection;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;


namespace Produmex.Sbo.Logex.Servofarma
{
    static class Program
    {
        const string CONNECTION_STRING = "Server=localhost;CompanyDB=SBODemo_US;ExtraDb=PMX_SBODemo_US;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=produmex;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2005;DBCultureInfo=en-US;LicenseServer=localhost:30000";

        /// <summary>
        /// Used for logging
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        static void Main(string[] args)
        {
            LogProvider.SetConsoleAppender("W");

            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

                    // Manual UDF's
                    SboUserFieldMDProvider userFieldProv = new SboUserFieldMDProvider(conn);
                    using (SboUserFieldMD userField = userFieldProv.GetBO(PmxSalesOrderSFTable.NAME, PmxSalesOrderSFTable.Columns.AutoCreatePickWaves.NAME))
                    {
                        if (userField == null)
                        {
                            SboColumnDefinitionAttribute boolColumnDef = new SboColumnDefinitionAttribute(PmxSalesOrderSFTable.Columns.AutoCreatePickWaves.NAME,
                                PmxSalesOrderSFTable.Columns.AutoCreatePickWaves.DESCR,
                                PmxSalesOrderSFTable.Columns.AutoCreatePickWaves.TYPE,
                                PmxSalesOrderSFTable.Columns.AutoCreatePickWaves.SIZE,
                                PmxSalesOrderSFTable.Columns.AutoCreatePickWaves.SUB_TYPE);
                            boolColumnDef.SboBOColumnConverterType = typeof(SboBooleanConvertor);
                            boolColumnDef.DefaultValue = SboBooleanColumnValues.Y.VALUE;
                            userFieldProv.AddUpdateUserfield("ORDR", boolColumnDef);
                        }
                        
                    }

                    // UDT's
                    Console.WriteLine("Add update UDT PmxDocumentSeriesSF");
                    AddUpdateUserDefinedTable(typeof(PmxDocumentSeriesSF), conn);

                    //Update extensions
                    Console.WriteLine("Add update Config data");

                    AddUpdateExtension(
                        Produmex.Sbo.Logex.Servofarma.BusinessObjects.Definitions.PmxExtensionCodesSF.DOCUMENT_SERIES_CONTROLLER_SF,
                        "IDocumentSeriesController - Servofarma",
                        typeof(Produmex.Sbo.Logex.Servofarma.Generators.DocumentSeriesController.ServofarmaDocumentSeriesController),
                        Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionTypeCodes.IDOCUMENTSERIESCONTROLLER,
                        conn
                        );

                    AddUpdateExtension(
                        Produmex.Sbo.Logex.Servofarma.BusinessObjects.Definitions.PmxExtensionCodesSF.PICKLIST_ROBOT_CONTROLLER_SF,
                        "IPickListRobot - Servofarma",
                        typeof(Produmex.Sbo.Logex.Servofarma.Generators.PickListRobot.PickListRobotControllerSF),
                        Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionTypeCodes.IPICKLISTROBOT,
                        conn
                        );

                    
                                        
                }

                //Complete transaction
                scope.Complete();               
            }

            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

                    Console.WriteLine("Add update Form for PmxDocumentSeriesSF");
                    AddUpdateUserDefinedObject(typeof(PmxDocumentSeriesSF), conn);
                }

                //Complete transaction
                scope.Complete(); 
            }
        }

        private static void AddUpdateUserDefinedTable(Type type, SboConnection conn)
        {
            SboUserTableMDProvider userTableMDProv = new SboUserTableMDProvider(conn);
            userTableMDProv.AddUpdateBO(type);
        }

        private static void AddUpdateUserDefinedObject(Type type, SboConnection conn)
        {
            SboUserObjectMDProvider userObjectMDProv = new SboUserObjectMDProvider(conn);
            userObjectMDProv.AddUpdateBO(type);
        }

        
       

        /// <summary>
        /// Add/Update <see cref="PmxExtension"/>
        /// </summary>
        /// <param name="code">The code of the extension.</param>
        /// <param name="name">The name of the extension.</param>
        /// <param name="type">The type of the extension.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="conn">The SBO connection</param>
        private static void AddUpdateExtension(string code, string name, Type type, string extensionTypeCode, SboConnection conn)
        {
            PmxExtensionProvider prov = new PmxExtensionProvider(conn);
            PmxExtension extension = null;
            try
            {
                bool newBO = false;
                extension = prov.GetBO(code);
                if (extension == null)
                {
                    newBO = true;
                    extension = prov.GetNewBO();
                    extension.Code = code;
                }
                extension.Name = name;
                extension.ExtensionTypeCode = extensionTypeCode;
                extension.Type = ReflectionHelper.GetTypeAndAssemblyName(type);
                if (newBO)
                {
                    prov.AddBO(extension);
                }
                else
                {
                    prov.UpdateBO(extension);
                }
            }
            finally
            {
                if (extension != null) extension.Dispose();
            }
        }

        


    }
}
